==> release.12.2.28.1.tgz: cgi-cpn/prontus_port_regen_real.pl <==
#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# -------------------------------BEGIN SCRIPT--------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use Artic;
use DBI;
use Digest::MD5;
use File::Copy;
use HTML::Entities;
use JSON;
use POSIX;
use Time::Local;
use URI::Escape;
use glib_dbi_03;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_bd;
use lib_logproc;
use lib_maxrunning;
use lib_parser;
use lib_ports;
use lib_prontus;
use lib_quota;
use lib_tags;
use lib_tax;
use lib_urlf5;
use lib_utils;
use lib_waitlock;
##/libs##

my %PORTS_EDIC;
my %BASE_PORTS;
my $DIR_EDICS;
my $DIR_EDICS_TPL;
my $TABULADOR = "        ";

# ---------------------------------------------------------------
# MAIN.
# -------------
my %FORM;
my $TOT_REGS;
my $TOT_REGS_CON_ERR;
my $EDIC;
my $DST_TSEC;

main : {
    lib_utils::setUtf8();

    if (scalar @ARGV < 1) {
        print "Parámetros insuficientes.\n";
        exit;
    }

    $prontus_varglb::PRONTUS_ID = $ARGV[0];
    $FORM{'check_pp'}  = $ARGV[1];
    $FORM{'operador'}  = $ARGV[2];
    $FORM{'cmb_edic'}  = $ARGV[3];

    # valida el identificador de prontus
    if ($prontus_varglb::PRONTUS_ID eq '' || !lib_utils::valida_prontus($prontus_varglb::PRONTUS_ID)) {
        print "El identificador de Prontus es inválido\n";
        exit;
    }

    # Carga variables de configuracion.
    my $msg_load = lib_utils::load_config_by_id($prontus_varglb::PRONTUS_ID);
    if ($msg_load ne '') {
        print $msg_load , "\n";
        exit;
    }

    lib_utils::write_log('Actualiz. masiva', 'Portadas', '');

    # Establece log file
    $lib_logproc::LOG_FILE = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/prontus_port_regen_log.html";
    $lib_logproc::JS_FILE = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/result_port_regen.json";
    $lib_logproc::MODO_WEB = 0;

    if (lib_maxrunning::maxExcedido(1)) {
        lib_logproc::finishLoading("Ya se está ejecutando el proceso de regeneración de portadas, por favor espere.", 1);
        exit;
    }

    # Init
    lib_logproc::flush_log();
    lib_logproc::writeRule();
    lib_logproc::add_to_log_count("INICIANDO PROCESO DE ACTUALIZACION DE PORTADAS");

    $DIR_EDICS = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC";
    $DIR_EDICS_TPL = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'}$prontus_varglb::DIR_EDIC";

    &valida_param();
    &carga_portadas();

    # print STDERR "Procesando... $FORM{'operador'} $FORM{'cmb_edic'}\n";
    my (@ediciones) = &get_edics2update($FORM{'cmb_edic'}, $FORM{'operador'});
    foreach my $edic (@ediciones) {
        &regenerar_portadas($edic);
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5') {
        # ejecutar regeneracion de links simbolicos si el sitio tiene friendly 5
        my $cmd = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/utils/prontus_urlf5_regen.pl $prontus_varglb::PRONTUS_ID port >/dev/null 2>&1";
        print STDERR "[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n";
        system $cmd;
    }

    # Borra cache de listas de articulos del cpan
    glib_fildir_03::borra_dir("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/data/cache");

    lib_logproc::add_to_log_count("PROCESO DE ACTUALIZACION DE PORTADAS FINALIZADO");
    lib_logproc::writeRule();

    $TOT_REGS = '0' if ($TOT_REGS eq '');
    $TOT_REGS_CON_ERR = '0' if ($TOT_REGS_CON_ERR eq '');

    print STDERR "TOT_REGS[$TOT_REGS]\n";
    print STDERR "TOT_REGS_CON_ERR[$TOT_REGS_CON_ERR]\n";

    lib_logproc::add_to_log("Nro. de portadas procesadas: $TOT_REGS\nRegistros con Errores: $TOT_REGS_CON_ERR");
    lib_logproc::add_to_log_finish("Operaci&oacute;n finalizada.");

    lib_logproc::updateStatus(1, "Proceso terminado", 1, "Nro. de registros procesados: $TOT_REGS Registros con Errores: $TOT_REGS_CON_ERR");
} # main

# -------------------------------------------------------------------------------------------------#
# SUB-RUTINAS.
# -------------
sub valida_param {

    if($FORM{'cmb_edic'} ne '' && $FORM{'cmb_edic'} ne 'base' && $FORM{'cmb_edic'} !~ /\d+_\d+_\d+_\d+/) {
        lib_logproc::finishLoading("Error: Edición indicada no es válida");
        lib_logproc::handle_error("Error: Edición indicada no es válida");
    }

    if(! -d "$DIR_EDICS/$FORM{'cmb_edic'}") {
        lib_logproc::finishLoading("Error: La edición indicada no existe");
        lib_logproc::handle_error("Error: La edición indicada no existe");
    }

    if($FORM{'operador'} ne '' && $FORM{'operador'} !~ /igual|mayor|menor/) {
        lib_logproc::finishLoading("Error: El parámetro operador no es válido");
        lib_logproc::handle_error("Error: El parámetro operador no es válido");
    }
}

# -------------------------------------------------------------------------------------------------#
sub get_edics2update {
    my $cmb_edic = shift;
    my $operador = shift;
    my @ediciones;

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} ne 'SI') {
        lib_logproc::add_to_log_count("No hay multiedición, solo se procesa edición [base]");
        push @ediciones, 'base';
        return @ediciones;
    }

    if ($cmb_edic eq '') {
        lib_logproc::add_to_log_count("Se toman ediciones base, vigente y ultima");
        @ediciones = lib_ports::get_edics4update();

    } else {
        if ($operador ne 'mayor' && $operador ne 'menor') {
            lib_logproc::add_to_log_count("Se toma exactamente la edición indicada [$cmb_edic]");
            push @ediciones, $cmb_edic;
            return @ediciones;
        } else {
            if ($operador eq 'menor') {
                lib_logproc::add_to_log_count("Se toman ediciones MENOR o igual a [$cmb_edic]");
            } else {
                lib_logproc::add_to_log_count("Se toman ediciones MAYOR o igual a [$cmb_edic]");
            }
        }

        # Abre directorio.
        opendir(DIR, $DIR_EDICS) || die "Can't opendir" . $DIR_EDICS . $!;
        my @entries = readdir(DIR);
        closedir DIR;
        # Descendentemente.
        my @totentries = sort {$b cmp $a} (@entries);

        my $nro_elem = 0;
        foreach my $edic (@totentries) {
            next if(substr($edic, 0, 1) eq '.');
            next unless((-d("$DIR_EDICS/$edic")) and ($nro_elem < $prontus_varglb::NRO_EDICS_WORK));
            if ($operador eq 'menor') {
                if (($edic cmp $cmb_edic) > 0) {
                    # print STDERR "$edic (descartada)\n";
                    next;
                }
            } else {
                if (($edic cmp $cmb_edic) < 0) {
                    # print STDERR "$edic (descartada)\n";
                    next;
                }
            }
            # print STDERR "$edic (push)\n";
            push @ediciones, $edic;
            $nro_elem = $nro_elem + 1;
        }
    }
    return @ediciones;
}
# -------------------------------------------------------------------------------------------------#
# Regenera todas las portadas publicadas, segun las ediciones indicadas
sub regenerar_portadas {
    my $edic = $_[0];

    lib_logproc::add_to_log_count("Procesando Edicion [$edic]");

    if(!-d "$DIR_EDICS/$edic") {
        # Si no hay carpeta, no hay xmls... no se puede hacer nada
        lib_logproc::add_to_log_count("${TABULADOR}Error: No existe la carpeta de la edición [$edic]");
        return;
    }

    my %port2process;
    # Selector para ver que portadas se actualizaran
    if($edic eq 'base') {
        %port2process = %BASE_PORTS;
    } else {
        %port2process = %PORTS_EDIC;
    }

    foreach my $portada (sort keys %port2process) {
        my $buffer = &get_xml_buffer($portada, $edic);
        if ($buffer eq '') {
            $TOT_REGS_CON_ERR++;
            next;
        } else {
            lib_logproc::add_to_log_count("${TABULADOR}Regenerando portada [$portada]");
            $TOT_REGS++;
        }

        # Si no existe el tag en el xml se asume como publicada.
        my $no_write = 0;

        if ($buffer =~ /<port_pub>(\d+)<\/port_pub>/) {
            my $port_pub = $1;

            if ($port_pub eq '1') { # la portada esta publicada.
                $no_write = 0; # desactiva el no_write de la portada.
            } else {
                $no_write = 1; # activa el no_write de la portada.
            }
        }

        # Rescatar la info de c/artic de la seccion correspondiente
        while ($buffer =~ /<rowartic>\s*<dir>(\d+)<\/dir>\s*<file>(\d+)<\/file>\s*<area>(\d+)<\/area>\s*<ord>(\d+)<\/ord>\s*<vb>(\d*)<\/vb>\s*<pub>(\d*)<\/pub>\s*<\/rowartic>/isg) {
            my ($dirfecha, $art, $area, $prio, $vb, $pub) = ($1, $2, $3, $4, $5, $6);

            # Asocia area al articulo.
            $lib_ports::AREA{$prontus_varglb::PRONTUS_ID}{$portada}{$art} = $area;
            # Asocia prioridad correspondiente.
            $lib_ports::PRIO{$prontus_varglb::PRONTUS_ID}{$portada}{$art} = $prio;
            if ($vb eq '') { $vb = 1; };
            # Asocia VoBo correspondiente.
            $lib_ports::VB{$prontus_varglb::PRONTUS_ID}{$portada}{$art} = $vb;
            $lib_ports::DIR_FECHA{$prontus_varglb::PRONTUS_ID}{$portada}{$art} = $dirfecha;
        }

        my $EDIC_SITE = "$DIR_EDICS/$edic/port";
        my $EDIC_TMPL = "$DIR_EDICS_TPL/nroedic/port";

        # Primero para la vista por defecto (o sea, sin vista)
        my $mv = '';
        my $sin_regen_xml = 0;
        my $ts_preview = '';
        lib_ports::make_portada("$EDIC_SITE/$portada", "$EDIC_TMPL/$portada", $prontus_varglb::DIR_SERVER, $prontus_varglb::PRONTUS_ID,
                               $mv, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'},
                               $EDIC, $sin_regen_xml, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'}, $ts_preview, $no_write);

        # Ahora proceso multivistas
        # para no reescribir el xml
        $sin_regen_xml = 1;
        $ts_preview = '';
        foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTAS'}}) {
            lib_ports::make_portada("$EDIC_SITE/$portada", $DST_TSEC . "$EDIC_TMPL/$portada", $prontus_varglb::DIR_SERVER, $prontus_varglb::PRONTUS_ID,
                                     $mv, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'},
                                     $EDIC, $sin_regen_xml, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'}, $ts_preview, $no_write);
        }

        lib_utils::write_log('Actualizar', 'Portada (regen)', "$EDIC_SITE/$portada");

        %lib_ports::AREA = ();
        %lib_ports::PRIO = ();
        %lib_ports::VB = ();
        %lib_ports::DIR_FECHA = ();

        if($FORM{'check_pp'} eq 'si') {
            &exec_postproceso("$EDIC_SITE/$portada", "$EDIC_TMPL/$portada");
        }
    }
}

# ---------------------------------------------------------------
# POST-PROCESO
# Una vez escrito la portada, ubica el nombre del script de post-proceso (con extension y con path absoluto completo) y lo ejecuta.
# Script de postproceso es optativo y puede ir en cualquier parte de la portada
sub exec_postproceso {
    my $port_site = shift;
    my $port_tmpl = shift;
    # LEE PORT
    my $buffer = glib_fildir_03::read_file($port_tmpl);
    $buffer =~ s/\r/\n/g;

    my $dir_macros = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/plantillas/edic/nroedic/macros";
    $buffer = lib_parser::add_macros($buffer, $dir_macros);
    my ($post_proceso, $postProcesoLista);

    while ($buffer =~ /<!--POST_PROCESO=(.+?)-->/sg) {
        $postProcesoLista .= "$1\t";
    }

    # Ejecuta en segundo plano el proceso pasandole x parametros el path completo a la portada.
    my @postProcesos = split(/\t/, $postProcesoLista);

    foreach my $pp (@postProcesos) {
        # para que sea un script valido debe ubicarse en el mismo dir. de cgi del prontus
        # quita ../, no se pueden usar directorios relativos hacia arriba
        $pp =~ s/\.\.\///g;
        $pp =~ s/[#;&<>|~]+//g;

        if ($pp =~ /^\w/) {
            my $cmd = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/$pp $port_site $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'} >/dev/null 2>&1 &";
            print STDERR "[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n";
            system $cmd;
        }
    }
}
# -------------------------------------------------------------------------------------------------#
sub get_xml_buffer {
    my ($portada, $edic) = @_;
    my $xml = $portada;
    $xml =~ s/\.\w+$/.xml/;
    my $rutaxml = "$DIR_EDICS/$edic/xml/$xml";

    if (!-f $rutaxml) {
        lib_logproc::add_to_log_count("${TABULADOR}Error: No existe xml de portada [$xml]");
        return '';
    }

    my $buffer = glib_fildir_03::read_file($rutaxml);
    return $buffer;
}

# -------------------------------------------------------------------------------------------------#
# Obtiene de la lista de tpls. de portadas la primera cuyo nombre sin extension coincida con el q viene por param.
sub get_nom_port {
    my $port_xml = $_[0];
    my $dir_tpl_seccs = $DST_TSEC;

    $port_xml =~ s/\.xml$//;

    foreach my $entry (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}}) {
        if ($entry =~ /^$port_xml\.\w+$/) {
            if ((-s "$dir_tpl_seccs/$entry") and (! -d "$dir_tpl_seccs/$entry")) {
            # print "entry[$entry] - port_xml[$port_xml]\n";
                return $entry;
            }
        }
    }

    return '';
}

# -------------------------------------------------------------------------------------------------#
sub carga_portadas {
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} ne 'SI') {
        %BASE_PORTS = %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}};
        return;
    }

    foreach my $port (@{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'BASE_PORTS'}}) {
        $BASE_PORTS{$port} = 1;
    }

    foreach my $port (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}}) {
        next if($BASE_PORTS{$port});
        $PORTS_EDIC{$port} = 1;
    }
}

==> release.12.3.1.2.tgz: cgi-cpn/prontus_port_regen_real.pl <==
#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# -------------------------------BEGIN SCRIPT--------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use Artic;
use DBI;
use Digest::MD5;
use File::Copy;
use HTML::Entities;
use JSON;
use POSIX;
use Time::Local;
use URI::Escape;
use glib_dbi_03;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_bd;
use lib_encodes;
use lib_lang;
use lib_logproc;
use lib_maxrunning;
use lib_parser;
use lib_ports;
use lib_prontus;
use lib_quota;
use lib_tags;
use lib_tax;
use lib_urlf5;
use lib_utils;
use lib_waitlock;
##/libs##

my %PORTS_EDIC;
my %BASE_PORTS;
my $DIR_EDICS;
my $DIR_EDICS_TPL;
my $TABULADOR = "        ";
my %FORM;
my $TOT_REGS;
my $TOT_REGS_CON_ERR;
my $EDIC;
my $DST_TSEC;

main : {
    lib_utils::setUp();

    if (scalar @ARGV < 1) {
        print lib_lang::get_message('global_use_instructions');
        print "perl $0 <prontus_id> <si|no> <igual|mayor|menor> <d_d_d_d>\n";
        exit;
    }

    my $prontus_id = $ARGV[0];
    $FORM{'check_pp'}  = $ARGV[1];
    $FORM{'operador'}  = $ARGV[2];
    $FORM{'cmb_edic'}  = $ARGV[3];

    # valida el identificador de prontus
    if ($prontus_id eq '' || !lib_utils::valida_prontus($prontus_id)) {
        print lib_lang::get_message('global_invalid_prontus_id') . "\n";
        exit;
    }

    # Carga variables de configuracion.
    my $msg_load = lib_utils::load_config_by_id($prontus_id);
    if ($msg_load ne '') {
        print $msg_load , "\n";
        exit;
    }

    lib_utils::write_log(lib_lang::get_message('global_regen_label') , lib_lang::get_message('global_homepages'), '');

    # Establece log file
    $lib_logproc::LOG_FILE = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/prontus_port_regen_log.html";
    $lib_logproc::JS_FILE  = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/result_port_regen.json";

    if (lib_maxrunning::maxExcedido(1)) {
        lib_logproc::finishLoading(lib_lang::get_message("port_regen_real_already_running"), 1);
        exit;
    }

    # Init
    lib_logproc::updateStatus(1, lib_lang::get_message('global_starting_process'), 0, '');
    lib_logproc::writeRule();
    lib_logproc::add_to_log_count(lib_lang::get_message("port_regen_real_starting"));

    $DIR_EDICS = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC";
    $DIR_EDICS_TPL = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'}$prontus_varglb::DIR_EDIC";

    &valida_param();
    &carga_portadas();

    # print STDERR "Procesando... $FORM{'operador'} $FORM{'cmb_edic'}\n";
    my (@ediciones) = &get_edics2update($FORM{'cmb_edic'}, $FORM{'operador'});
    foreach my $edic (@ediciones) {
        &regenerar_portadas($edic);
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
        # ejecutar regeneracion de links simbolicos si el sitio tiene friendly 5
        my $cmd = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/utils/prontus_urlf5_regen.pl $prontus_varglb::PRONTUS_ID port >/dev/null 2>&1";
        print STDERR "[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n";
        system $cmd;
    }

    # Borra cache de listas de articulos del cpan
    lib_ports::borrar_json_artic_pubs($prontus_varglb::PRONTUS_ID);

    lib_logproc::add_to_log_count(lib_lang::get_message("port_regen_real_end"));
    lib_logproc::writeRule();

    $TOT_REGS = '0' if ($TOT_REGS eq '');
    $TOT_REGS_CON_ERR = '0' if ($TOT_REGS_CON_ERR eq '');

    print STDERR ("port_regen_real_total_registers", $TOT_REGS);
    print STDERR ("port_regen_real_total_with_errors", $TOT_REGS_CON_ERR);

    lib_logproc::add_to_log(lib_lang::get_interpolated_message('global_processed_status_error', $TOT_REGS, $TOT_REGS_CON_ERR));
    lib_logproc::add_to_log_finish(lib_lang::get_message('global_finalized'));

    lib_logproc::updateStatus(1,  lib_lang::get_message('global_process_finished'), 1);
} # main

sub valida_param {

    if($FORM{'cmb_edic'} ne '' && $FORM{'cmb_edic'} ne 'base' && $FORM{'cmb_edic'} !~ /\d+_\d+_\d+_\d+/) {
        my $error = lib_lang::get_message('port_regen_real_invalid_edition');
        lib_logproc::finishLoading($error);
        lib_logproc::handle_error($error);
    }

    if(! -d "$DIR_EDICS/$FORM{'cmb_edic'}") {
        my $error = lib_lang::get_message('port_regen_real_missing_edition');
        lib_logproc::finishLoading($error);
        lib_logproc::handle_error($error);
    }

    if($FORM{'operador'} ne '' && $FORM{'operador'} !~ /igual|mayor|menor/) {
        my $error = lib_lang::get_message("port_regen_real_invalid_parameter");
        lib_logproc::finishLoading($error);
        lib_logproc::handle_error($error);
    }
}

sub get_edics2update {
    my $cmb_edic = shift;
    my $operador = shift;
    my @ediciones;

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} ne 'SI') {
        lib_logproc::add_to_log_count(lib_lang::get_message("port_regen_real_processing_base"));
        push @ediciones, 'base';
        return @ediciones;
    }

    if ($cmb_edic eq '') {
        lib_logproc::add_to_log_count(lib_lang::get_message("port_regen_real_processing_base_current_last"));
        @ediciones = lib_ports::get_edics4update();

    } else {
        if ($operador ne 'mayor' && $operador ne 'menor') {
            lib_logproc::add_to_log_count(lib_lang::get_interpolated_message("port_regen_real_processing_exact_edition", $cmb_edic));
            push @ediciones, $cmb_edic;
            return @ediciones;
        } else {
            if ($operador eq 'menor') {
                lib_logproc::add_to_log_count(lib_lang::get_interpolated_message("port_regen_real_processing_lesser", $cmb_edic));
            } else {
                lib_logproc::add_to_log_count(lib_lang::get_interpolated_message("port_regen_real_processing_greater", $cmb_edic));
            }
        }

        # Abre directorio.
        opendir(DIR, $DIR_EDICS) || die "Can't opendir" . $DIR_EDICS . $!;
        my @entries = readdir(DIR);
        closedir DIR;
        # Descendentemente.
        my @totentries = sort {$b cmp $a} (@entries);

        my $nro_elem = 0;
        foreach my $edic (@totentries) {
            next if(substr($edic, 0, 1) eq '.');
            next unless((-d("$DIR_EDICS/$edic")) and ($nro_elem < $prontus_varglb::NRO_EDICS_WORK));
            if ($operador eq 'menor') {
                if (($edic cmp $cmb_edic) > 0) {
                    # print STDERR "$edic (descartada)\n";
                    next;
                }
            } else {
                if (($edic cmp $cmb_edic) < 0) {
                    # print STDERR "$edic (descartada)\n";
                    next;
                }
            }
            # print STDERR "$edic (push)\n";
            push @ediciones, $edic;
            $nro_elem = $nro_elem + 1;
        }
    }
    return @ediciones;
}

# Regenera todas las portadas publicadas, segun las ediciones indicadas
sub regenerar_portadas {
    my $edic = $_[0];

    lib_logproc::add_to_log_count(lib_lang::get_interpolated_message("port_regen_real_processing_edition", $edic));

    if(!-d "$DIR_EDICS/$edic") {
        # Si no hay carpeta, no hay xmls... no se puede hacer nada
        lib_logproc::add_to_log_count(lib_lang::get_interpolated_message("port_regen_real_edition_dir_missing", ${TABULADOR}, $edic));
        return;
    }

    my %port2process;
    # Selector para ver que portadas se actualizaran
    if($edic eq 'base') {
        %port2process = %BASE_PORTS;
    } else {
        %port2process = %PORTS_EDIC;
    }

    foreach my $portada (sort keys %port2process) {
        my $buffer = &get_xml_buffer($portada, $edic);
        if ($buffer eq '') {
            $TOT_REGS_CON_ERR++;
            next;
        } else {
            lib_logproc::add_to_log_count(lib_lang::get_interpolated_message("port_regen_real_regenerating_homepage", ${TABULADOR}, $portada));
            $TOT_REGS++;
        }

        # Si no existe el tag en el xml se asume como publicada.
        # por defecto no_write desactivado.
        my $no_write = 0;
        my $port_pub = lib_ports::get_port_pub($buffer);
        # Si la portada no esta publicada se activa el no_write.
        $no_write = 1 if ($port_pub eq '0');

        # Rescatar la info de c/artic de la seccion correspondiente
        my %rowartics = lib_ports::get_rowartics($buffer);
        # articulos publicados en la portada
        foreach my $art (keys %rowartics) {
            # Asocia area al articulo.
            $lib_ports::AREA{$prontus_varglb::PRONTUS_ID}{$portada}{$art} = $rowartics{$art}{'area'};
            # Asocia prioridad correspondiente.
            $lib_ports::ORDEN{$prontus_varglb::PRONTUS_ID}{$portada}{$art} = $rowartics{$art}{'ord'};
            # Asocia VoBo correspondiente.
            $lib_ports::VB{$prontus_varglb::PRONTUS_ID}{$portada}{$art} = $rowartics{$art}{'vb'};
            $lib_ports::ART_PRONTUS{$prontus_varglb::PRONTUS_ID}{$portada}{$art} = $rowartics{$art}{'prontus'};
        }

        my $EDIC_SITE = "$DIR_EDICS/$edic/port";
        my $EDIC_TMPL = "$DIR_EDICS_TPL/nroedic/port";

        # Primero para la vista por defecto (o sea, sin vista)
        my $mv = '';
        my $sin_regen_xml = 0;
        my $ts_preview = '';
        lib_ports::make_portada("$EDIC_SITE/$portada", "$EDIC_TMPL/$portada", $prontus_varglb::DIR_SERVER, $prontus_varglb::PRONTUS_ID,
                               $mv, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'},
                               $EDIC, $sin_regen_xml, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'}, $ts_preview, $no_write);

        # Ahora proceso multivistas
        # para no reescribir el xml
        $sin_regen_xml = 1;
        $ts_preview = '';
        foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}}) {
            lib_ports::make_portada("$EDIC_SITE/$portada", $DST_TSEC . "$EDIC_TMPL/$portada", $prontus_varglb::DIR_SERVER, $prontus_varglb::PRONTUS_ID,
                                     $mv, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'},
                                     $EDIC, $sin_regen_xml, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'}, $ts_preview, $no_write);
        }

        lib_utils::write_log(lib_lang::get_message('port_regen_real_label'), lib_lang::get_message('port_regen_real_regen_label'), "$EDIC_SITE/$portada");

        %lib_ports::AREA = ();
        %lib_ports::ORDEN = ();
        %lib_ports::VB = ();
        %lib_ports::ART_PRONTUS = ();

        if ($FORM{'check_pp'} eq 'si') {
            print STDERR "$portada, $edic\n";
            lib_ports::exec_postproceso($prontus_varglb::PRONTUS_ID, $portada, $edic);
        }
    }
}

sub get_xml_buffer {
    my ($portada, $edic) = @_;
    my $xml = $portada;
    $xml =~ s/\.\w+$/.xml/;
    my $rutaxml = "$DIR_EDICS/$edic/xml/$xml";

    if (!-f $rutaxml) {
        lib_logproc::add_to_log_count(lib_lang::get_interpolated_message("port_regen_real_xml_missing", ${TABULADOR}, $xml));
        return '';
    }

    return glib_fildir_03::read_file($rutaxml);
}

# Obtiene de la lista de tpls. de portadas la primera cuyo nombre sin extension coincida con el q viene por param.
sub get_nom_port {
    my $port_xml = $_[0];
    my $dir_tpl_seccs = $DST_TSEC;

    $port_xml =~ s/\.xml$//;

    foreach my $entry (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}}) {
        if ($entry =~ /^$port_xml\.\w+$/) {
            if ((-s "$dir_tpl_seccs/$entry") and (! -d "$dir_tpl_seccs/$entry")) {
            # print "entry[$entry] - port_xml[$port_xml]\n";
                return $entry;
            }
        }
    }

    return '';
}

sub carga_portadas {
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} ne 'SI') {
        %BASE_PORTS = %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}};
        return;
    }

    foreach my $port (@{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'BASE_PORTS'}}) {
        $BASE_PORTS{$port} = 1;
    }

    foreach my $port (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}}) {
        next if($BASE_PORTS{$port});
        $PORTS_EDIC{$port} = 1;
    }
}
